package cytoPrefuse;

import java.awt.Point;
import java.util.Arrays;

import prefuse.data.Edge;
import prefuse.data.Node;
import prefuse.visual.NodeItem;

public class SpaceGrid {
	private interface GridVisitor{
		void visitGridPoint (int x, int y, int edgePenalty, int nodeEdgePenalty, int error, Node edgeSource, Node edgeDest);
	}
	
	private class OccupyGrid implements GridVisitor{

		public void visitGridPoint(int x, int y, int edgePenalty, int nodeEdgePenalty, int error, Node edgeSource, Node edgeDest) {
			if (error == 0){
				centerGridCrossCount[x][y]++;
			}
			gridCrossCount[x][y]++;
			impossibleCrossings[x][y][edgeSource.getRow()]++;
			impossibleCrossings[x][y][edgeDest.getRow()]++;
						
			
		}	
	}
	
	private class VacateGrid implements GridVisitor{
	
		public void visitGridPoint(int x, int y,  int edgePenalty, int nodeEdgePenalty, int error, Node edgeSource, Node edgeDest) {
			if (error == 0){
				centerGridCrossCount[x][y] --;
			}
			gridCrossCount[x][y] --;
			impossibleCrossings[x][y][edgeSource.getRow()]--;
			impossibleCrossings[x][y][edgeDest.getRow()]--;
			
		}		
	}
	
	private class CountIntersections implements GridVisitor{
		public void visitGridPoint(int x, int y, int edgePenalty, int nodeEdgePenalty, int error, Node edgeSource, Node edgeDest) {
			int crossCount = 0;
			crossCount += gridCrossCount[x][y];
			crossCount -= impossibleCrossings[x][y][edgeSource.getRow()];
			crossCount -= impossibleCrossings[x][y][edgeDest.getRow()];				
			
			if (crossCount > 0)
				crossingScore += crossCount * edgePenalty;
		 	if (error ==0 ){
				if (occupiedNodes[x][y]){
					crossingScore += nodeEdgePenalty;
				}
			}
			
		}
	
	}
	
	public static long intersectionsTested=0;
	public static long bbIntersectionsTested =0;
	public static long totalCrossings =0;
	public static long testEvents=0;
	
	private boolean[][] occupiedNodes;
	private int[][] centerGridCrossCount;
	private int[][] gridCrossCount;
	private int[][][] impossibleCrossings;
	
   private OccupyGrid occupyGrid = new OccupyGrid();
   private VacateGrid vacateGrid = new VacateGrid();
   private CountIntersections countIntersections = new CountIntersections();
   
   
   private int crossingScore;
  
   private SearchGridLayout layout;
   
   
   SpaceGrid (int x, int y, int maxNodes, SearchGridLayout inSearchGridLayout) {
	   System.out.println ("Initializing SpaceGrid with ["+ x + ":" + y+"]");
		occupiedNodes = new boolean [x][y];
		centerGridCrossCount = new int [x][y];
		gridCrossCount = new int [x][y];
		impossibleCrossings = new int [x][y][maxNodes];
	
		for (int xi=0; xi <x; xi++){
				Arrays.fill (gridCrossCount[xi], 0);
				Arrays.fill (centerGridCrossCount[xi],0);
				Arrays.fill (occupiedNodes[xi],false);
		};
				
		for (int xi=0; xi <x; xi++)
			for (int yi=0; yi < y; yi++){		
				Arrays.fill(impossibleCrossings[xi][yi], 0);
			}
		layout = inSearchGridLayout;
   }
   
   public void checkHealth () {
	   //For debugging
	   for (int x =0; x < gridCrossCount.length; x++){
		   for (int y =0; y < gridCrossCount[x].length; y++){
			   if (gridCrossCount[x][y] < 0) {
				   System.out.println ("["+x+":"+y+"] has score " + gridCrossCount[x][y]);
			   }
		   }
	   }
   }
		
	
	public void addEdge (Edge e) {
			visitGridsCrossedByEdge(e, occupyGrid,0,0);	
	}
	
	public void deleteEdge (Edge e) {
			visitGridsCrossedByEdge(e, vacateGrid,0,0);
	
	}	

	private void visitGridsCrossedByEdge (Edge e, GridVisitor gv, int edgePenalty, int nodeEdgePenalty){
		Point p0 = layout.getPositionOfNode( e.getSourceNode());
		Point p1 = layout.getPositionOfNode(e.getTargetNode());
		
		Node edgeSource = e.getSourceNode();
		Node edgeDest = e.getTargetNode();
		if (edgeSource.getRow() != edgeDest.getRow()){
			
			int x0 = p0.x;
			int x1 = p1.x;
			int y0 = p0.y;
			int y1 = p1.y;
		
			boolean steep = Math.abs(y1 - y0) > Math.abs(x1 - x0);
		    if (steep){
		    	  int temp = x0;
		    	  x0 = y0;
		    	  y0 = temp;
		    	  
		    	  temp = x1;
		    	  x1 = y1;
		    	  y1 =temp;
		    	  
		    }
		    if (x0 > x1){
		    	
		    	int temp = x0;
		    	x0 = x1;
		    	x1 = temp;
		    	
		    	temp = y0;
		    	y0 = y1;
		    	y1 = temp;
		    	
		    }
		     int deltax = x1 - x0;
		     int deltay =Math.abs(y1 - y0);
		     
		     int error = 0;
		     int y = y0;
		     int ystep = (y0 < y1)? 1 : -1;
		    
		     for (int x = x0; x <= x1; x++ ) {
		    	 if (steep)
		    		  gv.visitGridPoint(y,x, edgePenalty, nodeEdgePenalty, error, edgeSource, edgeDest);
		    	 else 
		    		  gv.visitGridPoint(x,y, edgePenalty, nodeEdgePenalty, error,edgeSource, edgeDest);
		         error = error + deltay;
		         if (2*error >= deltax) {
		             y = y + ystep;
		             error = error - deltax;
		         }
		     }
		}
	}
	
	public void addNode (NodeItem n) {
		Point nPos = layout.getPositionOfNode(n);
		if (occupiedNodes [nPos.x][nPos.y]) {
			System.out.println( "ADD Node BAD");
		}	
		occupiedNodes [nPos.x][nPos.y] = true;
		
	}
	
	public void deleteNode (NodeItem n) {
		
		Point nPos = layout.getPositionOfNode(n);
		if (!occupiedNodes [nPos.x][nPos.y]) {
			System.out.println( "DELETE Node BAD");
		}
		occupiedNodes [nPos.x][nPos.y] = false;
	}

	
	public int countCrossings(Edge e, int edgePenalty, int nodeEdgePenalty) {
		crossingScore =0;
		visitGridsCrossedByEdge(e, countIntersections, edgePenalty, nodeEdgePenalty);
		return crossingScore;
	
	}
	
	public int countCrossings (Point p, int nodeEdgePenalty) {		
		int x = p.x;
		int y = p.y;
	
		int crossings =0;
		crossings += centerGridCrossCount[x][y] * nodeEdgePenalty;
		return crossings;
	}	
}
