package prefuse.controls;

import java.awt.Cursor;
import java.awt.event.MouseEvent;
import java.awt.geom.Rectangle2D;

import prefuse.Display;
import prefuse.util.ui.UILib;
import prefuse.visual.VisualItem;

/**
 * Pans the display, changing the viewable region of the visualization.
 * By default, panning is accomplished by clicking on the background of a
 * visualization with the left mouse button and then dragging.
 *
 * The panning is restricted so that the boundingGroup is never entirely panned offscreen
 * 
 * @author <a href="http://jheer.org">jeffrey heer</a>
 */
public class BoundedPanControl extends ControlAdapter  {
	    private int m_xDown, m_yDown;
	    private int m_button;
	    private double margin = 50.0;
	    private String boundingGroup;
	    
	    /**
	     * Create a new PanControl.
	     */
	    public BoundedPanControl(String boundingGroup) {
	        this(LEFT_MOUSE_BUTTON, boundingGroup);
	    }
	    /**
	     * Create a new PanControl
	     * @param mouseButton the mouse button that should initiate a pan. One of
	     * {@link Control#LEFT_MOUSE_BUTTON}, {@link Control#MIDDLE_MOUSE_BUTTON},
	     * or {@link Control#RIGHT_MOUSE_BUTTON}.
	     * @param boundingGroup the visual group that should always be visible.
	     */
	    public BoundedPanControl(int mouseButton, String boundingGroup) {
	        m_button = mouseButton;
	        this.boundingGroup = boundingGroup;
	    }
	    
	    // ------------------------------------------------------------------------
	    
	    /**
	     * @see java.awt.event.MouseListener#mousePressed(java.awt.event.MouseEvent)
	     */
	    public void mousePressed(MouseEvent e) {
	        if ( UILib.isButtonPressed(e, m_button) ) {
	        	Display d = (Display)e.getComponent();
	        	d.setCursor(
	                Cursor.getPredefinedCursor(Cursor.MOVE_CURSOR));
	            m_xDown = e.getX();
	            m_yDown = e.getY();
	            d.setHighQuality(false);
	        }
	    }
	    
	    /**
	     * @see java.awt.event.MouseMotionListener#mouseDragged(java.awt.event.MouseEvent)
	     */
	    public void mouseDragged(MouseEvent e) {
	        if ( UILib.isButtonPressed(e, m_button) ) {
	            Display display = (Display)e.getComponent();
	            int x = e.getX(),   y = e.getY();
	            int dx = x-m_xDown, dy = y-m_yDown;
	            Rectangle2D bounds = display.getVisualization().getBounds(boundingGroup);
	            double panx = dx / display.getScale();
	            double pany = dy / display.getScale();
	  
	            boolean widthOK;
	            if (panx < 0 ){
	            	widthOK =(display.getDisplayX()/display.getScale() - panx + margin/display.getScale()) < bounds.getWidth();
	            }else{
	            	widthOK =(display.getDisplayX()/display.getScale() - panx -margin/display.getScale() + display.getWidth()/display.getScale() > bounds.getMinX());
	            }
	            
	            boolean heightOK;
	            if (pany < 0) {
	            	heightOK = (display.getDisplayY()/display.getScale() - pany + margin/display.getScale()) < bounds.getHeight();
	            }else {
	             	heightOK =(display.getDisplayY()/display.getScale() - pany -margin/display.getScale() + display.getHeight()/display.getScale() > bounds.getMinY() );
	 	           
	            }
	            if (!widthOK)
	            	panx = 0;
	            if (!heightOK)
	            	pany = 0;
           		display.panAbs(panx,pany);
            	display.repaint();
           
	         	m_xDown = x;
            	m_yDown = y;
           
	        }
	    }
	    
	    /**
	     * @see java.awt.event.MouseListener#mouseReleased(java.awt.event.MouseEvent)
	     */
	    public void mouseReleased(MouseEvent e) {
	        if ( UILib.isButtonPressed(e, m_button) ) {
	        	Display d = (Display)e.getComponent();
	        	d.setCursor(Cursor.getDefaultCursor());
	        	d.setHighQuality(true);
	            m_xDown = -1;
	            m_yDown = -1;
	            d.repaint();
	        }
	    }
	    
	public void itemPressed(VisualItem item, MouseEvent e) {
	      //	if (item instanceof EdgeItem)
	            mousePressed(e);
	    }

	    /**
	     * @see prefuse.controls.Control#itemDragged(prefuse.visual.VisualItem, java.awt.event.MouseEvent)
	     */
	    public void itemDragged(VisualItem item, MouseEvent e) {
	    //	if (item instanceof EdgeItem)
	    		 mouseDragged(e);
	    }
	    
	    /**
	     * @see prefuse.controls.Control#itemReleased(prefuse.visual.VisualItem, java.awt.event.MouseEvent)
	     */
	    public void itemReleased(VisualItem item, MouseEvent e) {
	    //	if (item instanceof EdgeItem)
	    		 mouseReleased(e);
	    }
}
